@@ -24,10 +24,22 @@
 #ifndef __JSON_GLIB_H__
 #define __JSON_GLIB_H__
 
+#define __JSON_GLIB_INSIDE__
+
 #include <json-glib/json-types.h>
+
+#include <json-glib/json-builder.h>
 #include <json-glib/json-generator.h>
 #include <json-glib/json-parser.h>
+#include <json-glib/json-reader.h>
 #include <json-glib/json-version.h>
+
 #include <json-glib/json-enum-types.h>
 
+#include <json-glib/json-gobject.h>
+
+#include <json-glib/json-gvariant.h>
+
+#undef __JSON_GLIB_INSIDE__
+
 #endif /* __JSON_GLIB_H__ */
